Attribute VB_Name = "z_ParserRegex"
' ======== Parser regex masks ========
' Shared module version: 20210909
' Tested in:
' Depends on:
' Required reference:
Option Private Module
Option Explicit

Private Const P_SPACE = "(?:\s+?)"

Private Const P_QUOTE_OPEN = "[""']"
Private Const P_QUOTE_CLOSE = "[""']"
Private Const P_ANY_SHORTEST = ".*?"
Private Const P_ANY = ".*"

Private Const P_MONTH_RUS = "(?:|||||||||||)"
Private Const P_MONTH_DIGITS = "[0-1]?\d"
Private Const P_DAY_DIGITS = "[0-3]?\d"
Private Const P_YEAR_LONG = "[1-2]\d\d\d"
Private Const P_YEAR_SHORT = "\d\d"
Private Const P_YEAR_DIGITS = "(?:" & P_YEAR_LONG & "|" & P_YEAR_SHORT & ")"
  
Private Const P_DATE_TEXT = _
  "(" & P_DAY_DIGITS & ")" & P_SPACE & _
  "(" & P_MONTH_RUS & ")" & P_SPACE & _
  "(" & P_YEAR_LONG & ")"
  
Private Const P_DATE_NUMERIC = _
  "(" & P_DAY_DIGITS & ")" & "\." & _
  "(" & P_MONTH_DIGITS & ")" & "\." & _
  "(" & P_YEAR_DIGITS & ")"
  
Private Const P_NPA_PREFIX = "[--][\d--A-Za-z,\-\.""'\s]+?"
Private Const P_NPA_ID_ELEMENT = "[#N]" & P_SPACE & "?" & "[\d--A-Za-z_@\.\-\\/]+"
Private Const P_NPA_ID_SEQ = P_NPA_ID_ELEMENT & "(?:," & P_SPACE & P_NPA_ID_ELEMENT & ")*"
Private Const P_NPA_INTRO = _
  "(?:" & _
    "[] " & "|" & _
    "[] " & "|" & _
    "(?:" & _
      "[][]?" & "|" & _
      "(?:" & _
        "[]\." & "|" & _
        "[](?:[]|||)" & _
      ")" & "  (?:[]|||)  " & "|" & _
      "(?:[]?  )?" & _
      "(?:" & _
        "[]\." & "|" & _
        "[](?:[]|||)" & "|" & _
        "[]?" & "|" & _
        "[](?:[]|||)  " & _
      ")" & "|" & _
      "[]?" & _
    ")" & "(?: )?" & _
  ")"

Private Const P_MASK_DOCTYPE = _
  "(?:" & _
    "[][]" & "|" & _
    "[](?:|||) (?:||)?" & "|" & _
    "[](?:|||) [](?:|||)] (?:||)?" & "|" & _
    "[](?:||)?(?: |  )?" & "|" & _
    "[](?:|||)" & "|" & _
    P_ANY & "[](?:)?" & P_ANY & "|" & _
    P_ANY & "[](?:||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:||)?" & "|" & _
    "[](?:||)?" & "|" & _
    "[](?:|||) (?:|||)" & "|" & _
    "[](?:|||)" & "|" & _
    "[](?:|||)(?:  )?" & "|" & _
    "[](?:|||)" & _
  ")"

Private Const P_NPA_TITLE_PREFIX = _
  "(?:" & _
    "[]   " & "(?:  )?" & "|" & _
    "[]   " & "|" & _
    "[] (?: (?: )?   )?" & _
  " " & _
  ")"

' ======================================
' ========   ==========
' ======================================

' =======  ==========
Public Const P_DATE_NO_CAPTURE = _
  "(?:" & _
    P_DAY_DIGITS & P_SPACE & P_MONTH_RUS & P_SPACE & P_YEAR_LONG & _
    "|" & _
    P_DAY_DIGITS & "\." & P_MONTH_DIGITS & "\." & P_YEAR_DIGITS & _
  ")" & _
  "(?:" & P_SPACE & "?" & "(?:|\.|))?"
  
Public Const P_DATE_CAPTURE = _
  "^" & _
    "(?:" & _
      P_DATE_TEXT & "|" & _
      P_DATE_NUMERIC & _
    ")" & _
    "(?:" & P_SPACE & "?" & "(?:|\.|))?" & _
  "$"
  
' ======   ======
Public Const P_NPA_SCAN = _
  "(?:" & "\b[]\." & P_SPACE & ")?" & _
  "(?:" & _
      "\b  " & "|" & _
      "\b   " & _
  ")?" & _
  "(" & _
    P_NPA_PREFIX & P_SPACE & "" & P_SPACE & P_DATE_NO_CAPTURE & P_SPACE & "?" & _
    P_NPA_ID_SEQ & "(?:" & P_SPACE & P_QUOTE_OPEN & P_ANY & P_QUOTE_CLOSE & ")?" & _
    "|" & _
    P_ANY_SHORTEST & _
    "\(" & _
      P_ANY_SHORTEST & P_NPA_INTRO & P_SPACE & P_NPA_PREFIX & _
      "(?:" & P_SPACE & "" & ")?" & P_SPACE & P_DATE_NO_CAPTURE & _
      "(?:" & P_SPACE & "?" & "(?:" & P_NPA_ID_SEQ & "))?" & P_ANY_SHORTEST & _
    "\)" & _
  ")"

Public Const P_NPA_PROPS_FIRST = _
  "^" & _
    "(" & P_NPA_PREFIX & ")" & _
    P_SPACE & "" & P_SPACE & "(" & P_DATE_NO_CAPTURE & ")" & _
    P_SPACE & "?" & "(" & P_NPA_ID_SEQ & ")" & _
    "(?:" & P_SPACE & P_QUOTE_OPEN & "(" & P_ANY & ")" & P_QUOTE_CLOSE & ")?" & _
    P_ANY_SHORTEST & _
  "$"

Public Const P_NPA_TITLE_FIRST = _
  "^" & _
    "(" & P_ANY_SHORTEST & ")" & _
    "\(" & _
      P_ANY_SHORTEST & P_NPA_INTRO & P_SPACE & _
      "(" & P_NPA_PREFIX & ")" & P_SPACE & _
      "(?:" & P_SPACE & ")?" & "(" & P_DATE_NO_CAPTURE & ")" & _
      "(?:" & P_SPACE & "?" & "(" & P_NPA_ID_SEQ & "))?" & _
      P_ANY_SHORTEST & _
    "\)" & _
    P_ANY_SHORTEST & _
  "$"
  
' =====   =========
Public Const P_NPA_DOCTYPE = "^" & "(" & P_MASK_DOCTYPE & ")"

' ========   ======
Public Const P_NPA_COMPOSITE_TITLE = _
  "^" & _
    "(" & P_NPA_TITLE_PREFIX & ")" & _
    P_QUOTE_OPEN & "?" & _
    "(" & P_ANY_SHORTEST & ")" & _
    P_QUOTE_CLOSE & "?" & _
  "$"

' ========   ===
Public Const P_NPA_IMMEDIATE = _
  " " & _
  "(?:(?:|||) |\()" & _
  "(?: | |  )?" & _
  P_YEAR_DIGITS & _
  "(?:" & _
    "(?:\-| \- |  |  )" & _
    P_YEAR_DIGITS & _
  ")?" & _
  " " & _
  "(?:(?:|||)?|\.||\.|)"

' ======== Static access for compiled regexp ======
Public Function GlobalDateRegex() As RegExp
  Static s_Regex As RegExp
  If s_Regex Is Nothing Then
    Set s_Regex = New RegExp
    s_Regex.Global = True
    s_Regex.Pattern = P_DATE_NO_CAPTURE
  End If
  Set GlobalDateRegex = s_Regex
End Function

Public Function GlobalNPARegex() As RegExp
  Static s_Regex As RegExp
  If s_Regex Is Nothing Then
    Set s_Regex = New RegExp
    s_Regex.Global = True
    s_Regex.Pattern = P_NPA_SCAN
  End If
  Set GlobalNPARegex = s_Regex
End Function

' =============  -
' (?:||) ?
' (?:||)
' ?
' (?:)?[]?(?: )?
' [] 
' []
' []
' (?:|) [] ?
' (?:(?:, )? )?
' []?  
' []?
' 

' ============  -
'    
' (?: | | )?   (?:||)
'   
'  
'   
'  (||)
'  
'  (?:  )?
'   
'   ' -
'   '  
'  (?: ? )?
'  
'   ' = c?
'   '  
'   '   

' ========= TODO
'   -
' Private Const P_ANY_SPECIAL_SPACE = "[ \xA0\u1680\u180E\u2000-\u200B\u202F\u205F\u3000\uFEFF]+"

